@@ -1,6 +1,9 @@
+from __future__ import annotations
+
+from collections.abc import Callable
+from collections.abc import Sequence
 import errno
 import os
-import typing
 import warnings
 
 import numpy as np
@@ -10,17 +13,13 @@ import audeer
 import audinterface
 import audobject
 
-from opensmile.core.SMILEapi import (
-    OpenSMILE,
-    FrameMetaData,
-)
 from opensmile.core.config import config
-from opensmile.core.define import (
-    FeatureLevel,
-    FeatureLevelResolver,
-    FeatureSet,
-    FeatureSetResolver,
-)
+from opensmile.core.define import FeatureLevel
+from opensmile.core.define import FeatureLevelResolver
+from opensmile.core.define import FeatureSet
+from opensmile.core.define import FeatureSetResolver
+from opensmile.core.lib import FrameMetaData
+from opensmile.core.lib import OpenSMILE
 
 
 class Smile(audinterface.Feature, audobject.Object):
@@ -92,8 +91,7 @@ class Smile(audinterface.Feature, audobject.Object):
         multiprocessing: use multiprocessing instead of multithreading
         verbose: show debug messages
 
-    Example:
-
+    Examples:
     >>> sampling_rate = 16000
     >>> signal = np.zeros(sampling_rate)
     >>> smile = Smile(
@@ -105,59 +103,55 @@ class Smile(audinterface.Feature, audobject.Object):
     0 days  0 days 00:00:01    0.0
     Name: audspec_lengthL1norm_sma_range, dtype: float32
 
-    .. _`hidden arguments`: http://tools.pp.audeering.com/audobject/usage.html#hidden-arguments
+    .. _`hidden arguments`: https://audeering.github.io/audobject/usage.html#hidden-arguments
 
     """  # noqa: E501
+
     @audobject.init_decorator(
         borrow={
-            'sampling_rate': 'process',
-            'channels': 'process',
-            'mixdown': 'process',
-            'resample': 'process',
+            "sampling_rate": "process",
+            "channels": "process",
+            "mixdown": "process",
+            "resample": "process",
         },
         hide=[
-            'keep_nat',
-            'logfile',
-            'loglevel',
-            'num_workers',
-            'multiprocessing',
-            'segment',
-            'verbose',
+            "keep_nat",
+            "logfile",
+            "loglevel",
+            "num_workers",
+            "multiprocessing",
+            "segment",
+            "verbose",
         ],
         resolvers={
-            'feature_set': FeatureSetResolver,
-            'feature_level': FeatureLevelResolver,
-        }
+            "feature_set": FeatureSetResolver,
+            "feature_level": FeatureLevelResolver,
+        },
     )
     @audeer.deprecated_keyword_argument(
-        deprecated_argument='num_channels',
-        removal_version='0.13.0',
-        new_argument='channels',
+        deprecated_argument="num_channels",
+        removal_version="0.13.0",
+        new_argument="channels",
         mapping=lambda x: range(x),
     )
     def __init__(
-            self,
-            feature_set: typing.Union[
-                str, FeatureSet
-            ] = FeatureSet.ComParE_2016,
-            feature_level: typing.Union[
-                str, FeatureLevel
-            ] = FeatureLevel.Functionals,
-            *,
-            options: dict = None,
-            loglevel: int = 2,
-            logfile: str = None,
-            sampling_rate: int = None,
-            channels: typing.Union[int, typing.Sequence[int]] = 0,
-            mixdown: bool = False,
-            resample: bool = False,
-            segment: audinterface.Segment = None,
-            keep_nat: bool = False,
-            num_workers: typing.Optional[int] = 1,
-            multiprocessing: bool = False,
-            verbose: bool = False,
+        self,
+        feature_set: str | FeatureSet = FeatureSet.ComParE_2016,
+        feature_level: str | FeatureLevel = FeatureLevel.Functionals,
+        *,
+        options: dict = None,
+        loglevel: int = 2,
+        logfile: str = None,
+        sampling_rate: int = None,
+        channels: int | Sequence[int] = 0,
+        mixdown: bool = False,
+        resample: bool = False,
+        segment: audinterface.Segment = None,
+        keep_nat: bool = False,
+        num_workers: int | None = 1,
+        multiprocessing: bool = False,
+        verbose: bool = False,
     ):
-
         self.feature_level = feature_level
         r"""Standard feature level or sink level in custom config file."""
         self.feature_set = feature_set
@@ -174,7 +168,7 @@ class Smile(audinterface.Feature, audobject.Object):
 
         super().__init__(
             self._feature_names(),
-            name='smile',
+            name="smile",
             params=None,
             process_func=self._extract,
             num_workers=num_workers,
@@ -208,7 +202,6 @@ class Smile(audinterface.Feature, audobject.Object):
     @property
     def config_path(self) -> str:
         r"""Return file path of config file."""
-
         if type(self.feature_set) is FeatureSet:
             config_path = os.path.join(
                 self.default_config_root,
@@ -244,7 +237,7 @@ class Smile(audinterface.Feature, audobject.Object):
                 )
 
     def _check_deprecated(self):
-        r"""Check if feature set is deprecated"""
+        r"""Check if feature set is deprecated."""
         deprecated_feature_sets = {  # deprecated: recommended
             FeatureSet.GeMAPS: FeatureSet.GeMAPSv01b,
             FeatureSet.GeMAPSv01a: FeatureSet.GeMAPSv01b,
@@ -252,8 +245,10 @@ class Smile(audinterface.Feature, audobject.Object):
             FeatureSet.eGeMAPSv01a: FeatureSet.eGeMAPSv02,
             FeatureSet.eGeMAPSv01b: FeatureSet.eGeMAPSv02,
         }
-        if type(self.feature_set) is FeatureSet and \
-                self.feature_set in deprecated_feature_sets:
+        if (
+            type(self.feature_set) is FeatureSet
+            and self.feature_set in deprecated_feature_sets
+        ):
             warnings.warn(
                 f"Feature set '{self.feature_set}' is "
                 f"deprecated, consider switching to "
@@ -262,12 +257,11 @@ class Smile(audinterface.Feature, audobject.Object):
             )
 
     def _extract(
-            self,
-            signal: np.ndarray,
-            sampling_rate: int,
+        self,
+        signal: np.ndarray,
+        sampling_rate: int,
     ) -> (pd.TimedeltaIndex, pd.TimedeltaIndex, np.ndarray):
         r"""Run feature extraction."""
-
         signal = signal.copy()
         signal *= 32768
         signal = signal.astype(np.int16)
@@ -277,37 +271,30 @@ class Smile(audinterface.Feature, audobject.Object):
         ends = []
 
         for x in signal:
-
             y = []
             starts = []
             ends = []
 
             options = self._options()
-            options['source'] = os.path.join(
-                self.default_config_root,
-                config.EXTERNAL_INPUT_CONFIG
+            options["source"] = os.path.join(
+                self.default_config_root, config.EXTERNAL_INPUT_CONFIG
             )
-            options['sampleRate'] = sampling_rate
-            options['nBits'] = 16
+            options["sampleRate"] = sampling_rate
+            options["nBits"] = 16
 
             smile = self._smile(options=options)
             smile.external_sink_set_callback_ex(
-                config.EXTERNAL_OUTPUT_COMPONENT,
-                Smile._sink_callback(y, starts, ends)
+                config.EXTERNAL_OUTPUT_COMPONENT, Smile._sink_callback(y, starts, ends)
             )
             smile.external_audio_source_write_data(
                 config.EXTERNAL_SOURCE_COMPONENT, bytes(x)
             )
-            smile.external_audio_source_set_eoi(
-                config.EXTERNAL_SOURCE_COMPONENT
-            )
+            smile.external_audio_source_set_eoi(config.EXTERNAL_SOURCE_COMPONENT)
             smile.run()
             smile.free()
 
             if not y:
-                warnings.warn(
-                    UserWarning("Segment too short, filling with NaN.")
-                )
+                warnings.warn(UserWarning("Segment too short, filling with NaN."))
                 y.append(np.ones(self.num_features) * np.nan)
                 starts.append(0)
                 ends.append(signal.size / sampling_rate)
@@ -315,32 +302,30 @@ class Smile(audinterface.Feature, audobject.Object):
             starts = np.vstack(starts).squeeze()
             ends = np.vstack(ends).squeeze()
             if starts.shape:
-                starts = pd.to_timedelta(starts, 's')
-                ends = pd.to_timedelta(ends, 's')
+                starts = pd.to_timedelta(starts, "s")
+                ends = pd.to_timedelta(ends, "s")
             else:
-                starts = pd.TimedeltaIndex([pd.to_timedelta(starts, 's')])
-                ends = pd.TimedeltaIndex([pd.to_timedelta(ends, 's')])
+                starts = pd.TimedeltaIndex([pd.to_timedelta(starts, "s")])
+                ends = pd.TimedeltaIndex([pd.to_timedelta(ends, "s")])
 
             y = np.vstack(y)
             ys.append(y)
 
         return starts, ends, np.concatenate(ys, axis=1)
 
-    def _feature_names(self) -> typing.List[str]:
+    def _feature_names(self) -> list[str]:
         r"""Read feature names from config file."""
         options = self._options()
-        options['source'] = os.path.join(
-            self.default_config_root,
-            config.EXTERNAL_INPUT_CONFIG
+        options["source"] = os.path.join(
+            self.default_config_root, config.EXTERNAL_INPUT_CONFIG
         )
         smile = self._smile(options=options)
         num_elements = smile.external_sink_get_num_elements(
             config.EXTERNAL_OUTPUT_COMPONENT
         )
         names = [
-            smile.external_sink_get_element_name(
-                config.EXTERNAL_OUTPUT_COMPONENT,
-                idx) for idx in range(num_elements)
+            smile.external_sink_get_element_name(config.EXTERNAL_OUTPUT_COMPONENT, idx)
+            for idx in range(num_elements)
         ]
         smile.free()
         return names
@@ -348,33 +333,34 @@ class Smile(audinterface.Feature, audobject.Object):
     def _options(self) -> dict:
         r"""Fill options dictionary."""
         options = self.options.copy()
-        options['sink'] = os.path.join(
-            self.default_config_root,
-            config.EXTERNAL_OUTPUT_SINGLE_CONFIG
+        options["sink"] = os.path.join(
+            self.default_config_root, config.EXTERNAL_OUTPUT_SINGLE_CONFIG
         )
         if type(self.feature_level) is FeatureLevel:
-            options['sinkLevel'] = self.feature_level.value
+            options["sinkLevel"] = self.feature_level.value
         else:
-            options['sinkLevel'] = self.feature_level
-        options['bufferModeRbConf'] = os.path.join(
-            self.default_config_root,
-            'shared/BufferModeRb.conf.inc'
+            options["sinkLevel"] = self.feature_level
+        options["bufferModeRbConf"] = os.path.join(
+            self.default_config_root, "shared/BufferModeRb.conf.inc"
         )
-        if 'frameModeFunctionalsConf' not in options:
-            options['frameModeFunctionalsConf'] = os.path.join(
-                self.default_config_root,
-                'shared/FrameModeFunctionals.conf.inc'
+        if "frameModeFunctionalsConf" not in options:
+            options["frameModeFunctionalsConf"] = os.path.join(
+                self.default_config_root, "shared/FrameModeFunctionals.conf.inc"
             )
         return options
 
     def _series_to_frame(
-            self,
-            series: pd.Series,
+        self,
+        series: pd.Series,
     ) -> pd.DataFrame:
-        r"""Usually, we need to figure out start and end times from
-        ``win_dur`` and ``hop_dur``. But since openSMILE provides segment
-        times, we can skip this step and use them directly."""
+        r"""Convert series to frame.
 
+        Usually, we need to figure out start and end times
+        from ``win_dur`` and ``hop_dur``.
+        But since openSMILE provides segment times,
+        we can skip this step and use them directly.
+
+        """
         frames = [None] * len(series)
         if len(series.index.levels) == 3:
             for idx, ((file, start, end), values) in enumerate(series.items()):
@@ -392,7 +378,7 @@ class Smile(audinterface.Feature, audobject.Object):
                         starts,
                         ends,
                     ],
-                    names=['file', 'start', 'end'],
+                    names=["file", "start", "end"],
                 )
                 frames[idx] = pd.DataFrame(
                     index=index,
@@ -412,14 +398,14 @@ class Smile(audinterface.Feature, audobject.Object):
                         starts,
                         ends,
                     ],
-                    names=['start', 'end'],
+                    names=["start", "end"],
                 )
                 frames[idx] = pd.DataFrame(
                     index=index,
                     data=values,
                     columns=self.column_names,
                 )
-        return pd.concat(frames, axis='index')
+        return pd.concat(frames, axis="index")
 
     def _smile(self, options: dict) -> OpenSMILE:
         r"""Set up smile instance."""
@@ -429,26 +415,27 @@ class Smile(audinterface.Feature, audobject.Object):
             options=options,
             loglevel=self.loglevel,
             log_file=self.logfile,
-            debug=self.verbose)
+            debug=self.verbose,
+        )
         return smile
 
     @staticmethod
     def _sink_callback(
-        y: typing.List[np.ndarray],
-        starts: typing.List[float],
-        ends: typing.List[float]
-    ) -> typing.Callable[[np.ndarray, FrameMetaData], None]:
+        y: list[np.ndarray], starts: list[float], ends: list[float]
+    ) -> Callable[[np.ndarray, FrameMetaData], None]:
         r"""Return callback where features are collected."""
+
         def callback(data: np.ndarray, meta: FrameMetaData):
             y.append(data.copy())
             starts.append(meta.time)
             ends.append(meta.time + meta.lengthSec)
+
         return callback
 
     def __call__(
-            self,
-            signal: np.ndarray,
-            sampling_rate: int,
+        self,
+        signal: np.ndarray,
+        sampling_rate: int,
     ) -> np.ndarray:
         r"""Apply processing to signal.
 
